/******************************************************************************
* File Name: main_cm4.c
*
* Version: 1.30
*
* Description: This example demonstrates the use cases of Watchdog Timer(WDT) 
* in PSoC 6 MCU — to reset the device in the case of a malfunction, as a Deep- 
* Sleep wakeup source, and for generating a periodic interrupt.
*
* Related Document: CE220060_PSoC6MCU_WatchdogTimer.pdf
*
* Hardware Dependency:  PSoC 6 BLE Pioneer Kit
*                       PSoC 6 WiFi-BT Pioneer Kit
*
******************************************************************************
* Copyright (2017), Cypress Semiconductor Corporation.
******************************************************************************
* This software, including source code, documentation and related materials
* ("Software") is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and 
* foreign), United States copyright laws and international treaty provisions. 
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the 
* Cypress source code and derivative works for the sole purpose of creating 
* custom software in support of licensee product, such licensee product to be
* used only in conjunction with Cypress's integrated circuit as specified in the
* applicable agreement. Any reproduction, modification, translation, compilation,
* or representation of this Software except as specified above is prohibited 
* without the express written permission of Cypress.
* 
* Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND, 
* EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes to the Software without notice. 
* Cypress does not assume any liability arising out of the application or use
* of Software or any product or circuit described in the Software. Cypress does
* not authorize its products for use as critical components in any products 
* where a malfunction or failure may reasonably be expected to result in 
* significant injury or death ("ACTIVE Risk Product"). By including Cypress's 
* product in a ACTIVE Risk Product, the manufacturer of such system or application
* assumes all risk of such use and in doing so indemnifies Cypress against all
* liability. Use of this Software may be limited by and subject to the applicable
* Cypress software license agreement.
*****************************************************************************/

#include "project.h"

/* WDT demo options */
#define WDT_RESET_DEMO						1
#define WDT_INTERRUPT_DEMO 					2
#define WDT_DEMO 							WDT_INTERRUPT_DEMO

/* WDT Interrupt Number */
#define WDT_IRQ__INTC_NUMBER 				srss_interrupt_IRQn
#define WDT_IRQ__INTC_CORTEXM4_PRIORITY 	7

/* WDT periodic interrupt timing */
#define WDT_INTERRUPT_INTERVAL				1000  //ms

/* Match count =  Desired interrupt interval in seconds x ILO Frequency in Hz */
#define WDT_MATCH_COUNT 					(WDT_INTERRUPT_INTERVAL*32000/1000)

#define LED_ON                              (0x00u)
#define LED_OFF                             (0x01u)

/* WDT interrupt configuration structure */
const cy_stc_sysint_t WDT_IRQ_cfg = {
	.intrSrc = (IRQn_Type)WDT_IRQ__INTC_NUMBER,
	.intrPriority = WDT_IRQ__INTC_CORTEXM4_PRIORITY
};

/* Function prototypes */
void InitializeWDT(void);
void WdtInterruptHandler(void);

/******************************************************************************
* Function Name: main
*******************************************************************************
*
* Summary: This is the main function for CM4 CPU. It configures the WDT in either
*          reset mode or periodic interrupt mode.
*
* Parameters:
*  None
*
* Return:
* int
*
******************************************************************************/
int main(void)
{
    /* Flash GREEN_LED_WDT_RESET after WDT reset event. */
    if (CY_SYSLIB_RESET_HWWDT == Cy_SysLib_GetResetReason())
    {                       
        Cy_GPIO_Write(GREEN_LED_WDT_RESET_0_PORT, GREEN_LED_WDT_RESET_0_NUM, LED_ON);        
        Cy_SysLib_Delay(200u);        
        Cy_GPIO_Write(GREEN_LED_WDT_RESET_0_PORT, GREEN_LED_WDT_RESET_0_NUM, LED_OFF);
    }
    /* Flash RED_LED_RESET after PowerUp or XRES */
    else
    {        
        Cy_GPIO_Write(RED_LED_RESET_0_PORT, RED_LED_RESET_0_NUM, LED_ON);
        Cy_SysLib_Delay(200u);
        Cy_GPIO_Write(RED_LED_RESET_0_PORT, RED_LED_RESET_0_NUM, LED_OFF);
    }

	/* Initialize WDT */
	InitializeWDT();

	/* Enable global interrupt */
	__enable_irq();

	for(;;)
	{
		#if(WDT_DEMO == WDT_RESET_DEMO)
			/* Clear WDT match event */
			Cy_WDT_ClearWatchdog();

			/* Uncomment this blocking code to cause device reset */
			//while(1);

			/* Constant delay of 1000ms */
			Cy_SysLib_Delay(1000);

			/* Invert the state of LED */
			Cy_GPIO_Inv(RED_LED_RESET_0_PORT, RED_LED_RESET_0_NUM);
		#endif

		#if(WDT_DEMO == WDT_INTERRUPT_DEMO)
			/* Uncomment below code to demonstrate wake up from deep sleep */
            //Cy_SysPm_DeepSleep(CY_SYSPM_WAIT_FOR_INTERRUPT);
		#endif
	}
}

/******************************************************************************
* Function Name: InitializeWDT
*******************************************************************************
*
* Summary: This function initializes the WDT block
*
* Parameters:
*  None
*
* Return:
*  None
*
******************************************************************************/
void InitializeWDT()
{
	 /* Step 1- Unlock WDT */
	Cy_WDT_Unlock();

	/* Step 2- Write the ignore bits - operate with full 16 bits */
	Cy_WDT_SetIgnoreBits(0);

	/* Step 3- Write match value */
	#if(WDT_DEMO == WDT_INTERRUPT_DEMO)
		Cy_WDT_SetMatch(WDT_MATCH_COUNT);
	#else
		Cy_WDT_SetMatch(0);
	#endif

	/* Step 4- Clear match event interrupt, if any */
	Cy_WDT_ClearInterrupt();

	/* Step 5- Enable ILO */
	Cy_SysClk_IloEnable();

	/* Step 6 - Enable interrupt if periodic interrupt mode selected */
	#if(WDT_DEMO == WDT_INTERRUPT_DEMO)
		Cy_SysInt_Init(&WDT_IRQ_cfg, WdtInterruptHandler);
		NVIC_EnableIRQ(WDT_IRQ_cfg.intrSrc);
		Cy_WDT_UnmaskInterrupt();
	#endif

	/* Step 7- Enable WDT */
	Cy_WDT_Enable();

	/* Step 8- Lock WDT configuration */
	Cy_WDT_Lock();
}

/******************************************************************************
* Function Name: WdtInterruptHandler
*******************************************************************************
*
* Summary: This function is the handler for the WDT interrupt
*
* Parameters:
*  None
*
* Return:
*  None
*
******************************************************************************/
void WdtInterruptHandler(void)
{
	/* Check if the interrupt is from WDT */
	if(SRSS->SRSS_INTR & SRSS_SRSS_INTR_WDT_MATCH_Msk)
	{
		/* Clear WDT Interrupt */
		Cy_WDT_ClearInterrupt();

		/* Update the match count */
		Cy_WDT_Unlock();

		/* Update the match count. Note that, if the ignore bits in the match is set
		 * to a non-zero value, the below code requires a modification. */
		Cy_WDT_SetMatch((uint16)(Cy_WDT_GetMatch() + WDT_MATCH_COUNT));
		Cy_WDT_Lock();

		/* Invert the state of LED */
		Cy_GPIO_Inv(BLUE_LED_WDT_INT_0_PORT, BLUE_LED_WDT_INT_0_NUM);
	}
}

/* [] END OF FILE */

